searchData={"items":[{"type":"module","title":"Canary","doc":"Canary provides plugs to use for authorization. The plug functions are defined in `Canary.Plugs`\n\nIn order to use the plugs, just `import Canary.Plugs`","ref":"Canary.html"},{"type":"module","title":"Canary.Plugs","doc":"Plug functions for loading and authorizing resources for the current request.\n\nThe plugs all store data in conn.assigns (in Phoenix applications, keys in conn.assigns can be accessed with `@key_name` in templates)\n\nIn order to use the plug functions, you must `use Canary`.\n\nYou must also specify the Ecto repo to use in your configuration:\n```\nconfig :canary, repo: Project.Repo\n```\nIf you wish, you may also specify the key where Canary will look for the current user record to authorize against:\n```\nconfig :canary, current_user: :some_current_user\n```\n\nYou can specify a handler function (in this case, `Helpers.handle_unauthorized`) to be called when an action is unauthorized like so:\n```elixir\nconfig :canary, unauthorized_handler: {Helpers, :handle_unauthorized}\n```\nor to handle when a resource is not found:\n```elixir\nconfig :canary, not_found_handler: {Helpers, :handle_not_found}\n```\nCanary will pass the `conn` to the handler function.","ref":"Canary.Plugs.html"},{"type":"function","title":"Canary.Plugs.authorize_controller/2","doc":"Authorize the current user against the calling controller.\n\nIn order to use this function,\n\n  1) `conn.assigns[Application.get_env(:canary, :current_user, :current_user)]` must be an ecto\n  struct representing the current user\n\n  2) `conn.private` must be a map (this should not be a problem unless you explicitly modified it)\n\nauthorize_controller checks for the name of the current controller in one of the following places\n  1) :phoenix_controller in conn.private\n  2) :canary_controller in conn.assigns\n\nIn case you are not using phoenix, make sure you set the controller name in the conn.assigns\nNote that in case neither of `:phoenix_controller` or `:canary_controller` are found the requested\n  authorization won't necessarily fail, rather it will trigger a `.can?` function with a `nil` controller\n\nIf authorization succeeds, sets `conn.assigns.authorized` to true.\n\nIf authorization fails, sets `conn.assigns.authorized` to false.\n\nOptional opts:\n\n* `:only` - Specifies which actions to authorize\n* `:except` - Specifies which actions for which to skip authorization\n* `:unauthorized_handler` - Specify a handler function to be called if the action is unauthorized\n\nExamples:\n```\nplug :authorize_controller\n\nplug :authorize_controller, only: [:index, :show]\n\nplug :authorize_controller, except: [:destroy]\n```","ref":"Canary.Plugs.html#authorize_controller/2"},{"type":"function","title":"Canary.Plugs.authorize_resource/2","doc":"Authorize the current user for the given resource.\n\nIn order to use this function,\n\n  1) `conn.assigns[Application.get_env(:canary, :current_user, :current_user)]` must be an ecto\n  struct representing the current user\n\n  2) `conn.private` must be a map (this should not be a problem unless you explicitly modified it)\n\nIf authorization succeeds, sets `conn.assigns.authorized` to true.\n\nIf authorization fails, sets `conn.assigns.authorized` to false.\n\nFor the `:index`, `:new`, and `:create` actions, the resource in the `Canada.Can` implementation\nshould be the module name of the model rather than a struct. A struct should be used instead of\nthe module name only if the `:persisted` key is used and you want to override the default\nauthorization behavior.  This can be useful when dealing with nested resources.\n\nFor example:\n\n  use\n  ```\n  def can?(%User{}, :index, Post), do: true\n  ```\n  instead of\n  ```\n  def can?(%User{}, :index, %Post{}), do: true\n  ```\n\n  or\n\n  use\n  ```\n  def can?(%User{id: user_id}, :index, %Post{user_id: user_id}), do: true\n  ```\n  if you are dealing with a nested resource, such as, \"/post/post_id/comments\"\n\n\n  You can specify additional actions for which Canary will authorize based on the model name, by passing the `non_id_actions` opt to the plug.\n\n  For example,\n  ```elixir\n  plug :authorize_resource, model: Post, non_id_actions: [:find_by_name]\n  ```\n\nRequired opts:\n\n* `:model` - Specifies the module name of the model to authorize access to\n\nOptional opts:\n\n* `:only` - Specifies which actions to authorize\n* `:except` - Specifies which actions for which to skip authorization\n* `:preload` - Specifies association(s) to preload\n* `:id_name` - Specifies the name of the id in `conn.params`, defaults to \"id\"\n* `:id_field` - Specifies the name of the ID field in the database for searching :id_name value, defaults to \"id\".\n* `:persisted` - Specifies the resource should always be loaded from the database, defaults to false\n* `:unauthorized_handler` - Specify a handler function to be called if the action is unauthorized\n\nExamples:\n```\nplug :authorize_resource, model: Post\n\nplug :authorize_resource, model: User, preload: :posts\n\nplug :authorize_resource, model: User, only: [:index, :show], preload: :posts\n\nplug :load_resource, model: Post, id_name: \"post_id\", only: [:index], persisted: true, preload: :comments\n\nplug :load_resource, model: Post, id_name: \"slug\", id_field: \"slug\", only: [:show], persisted: true\n```","ref":"Canary.Plugs.html#authorize_resource/2"},{"type":"function","title":"Canary.Plugs.load_and_authorize_resource/2","doc":"Authorize the given resource and then load it if\nauthorization succeeds.\n\nIf the resource cannot be loaded or authorization\nfails, conn.assigns.resource_name is set to nil.\n\nThe result of the authorization (true/false) is\nassigned to conn.assigns.authorized.\n\nAlso, see the documentation for load_resource/2 and\nauthorize_resource/2.\n\nRequired opts:\n\n* `:model` - Specifies the module name of the model to load resources from\n\nOptional opts:\n\n* `:as` - Specifies the `resource_name` to use\n* `:only` - Specifies which actions to authorize\n* `:except` - Specifies which actions for which to skip authorization\n* `:preload` - Specifies association(s) to preload\n* `:id_name` - Specifies the name of the id in `conn.params`, defaults to \"id\"\n* `:id_field` - Specifies the name of the ID field in the database for searching :id_name value, defaults to \"id\".\n* `:unauthorized_handler` - Specify a handler function to be called if the action is unauthorized\n* `:not_found_handler` - Specify a handler function to be called if the resource is not found\n\nNote: If both an `:unauthorized_handler` and a `:not_found_handler` are specified for `load_and_authorize_resource`,\nand the request meets the criteria for both, the `:unauthorized_handler` will be called first.\n\nExamples:\n```\nplug :load_and_authorize_resource, model: Post\n\nplug :load_and_authorize_resource, model: User, preload: :posts, as: :the_user\n\nplug :load_and_authorize_resource, model: User, only: [:index, :show], preload: :posts, as: :person\n\nplug :load_and_authorize_resource, model: User, except: [:destroy]\n\nplug :load_and_authorize_resource, model: Post, id_name: \"slug\", id_field: \"slug\", only: [:show], persisted: true\n```","ref":"Canary.Plugs.html#load_and_authorize_resource/2"},{"type":"function","title":"Canary.Plugs.load_resource/2","doc":"Load the given resource.\n\nLoad the resource with id given by `conn.params[\"id\"]` (or `conn.params[opts[:id_name]]` if `opts[:id_name]` is specified)\nand ecto model given by `opts[:model]` into `conn.assigns.resource_name`.\n\n`resource_name` is either inferred from the model name or specified in the plug declaration with the `:as` key.\nTo infer the `resource_name`, the most specific(right most) name in the model's\nmodule name will be used, converted to underscore case.\n\nFor example, `load_resource model: Some.Project.BlogPost` will load the resource into\n`conn.assigns.blog_post`\n\nIf the resource cannot be fetched, `conn.assigns.resource_name` is set\nto nil.\n\nBy default, when the action is `:index`, all records from the specified model will be loaded. This can\nbe overridden to fetch a single record from the database by using the `:persisted` key.\n\nCurrently, `:new` and `:create` actions are ignored, and `conn.assigns.resource_name`\nwill be set to nil for these actions. This can be overridden to fetch a single record from the database\nby using the `:persisted` key.\n\nThe `:persisted` key can override how a resource is loaded and can be useful when dealing\nwith nested resources.\n\nRequired opts:\n\n* `:model` - Specifies the module name of the model to load resources from\n\nOptional opts:\n\n* `:as` - Specifies the `resource_name` to use\n* `:only` - Specifies which actions to authorize\n* `:except` - Specifies which actions for which to skip authorization\n* `:preload` - Specifies association(s) to preload\n* `:id_name` - Specifies the name of the id in `conn.params`, defaults to \"id\"\n* `:id_field` - Specifies the name of the ID field in the database for searching :id_name value, defaults to \"id\".\n* `:persisted` - Specifies the resource should always be loaded from the database, defaults to false\n* `:not_found_handler` - Specify a handler function to be called if the resource is not found\n* `:required` - Same as `:persisted` but with not found handler - even for :index, :new or :create action\n\nExamples:\n```\nplug :load_resource, model: Post\n\nplug :load_resource, model: User, preload: :posts, as: :the_user\n\nplug :load_resource, model: User, only: [:index, :show], preload: :posts, as: :person\n\nplug :load_resource, model: User, except: [:destroy]\n\nplug :load_resource, model: Post, id_name: \"post_id\", only: [:new, :create], persisted: true\n\nplug :load_resource, model: Post, id_name: \"slug\", id_field: \"slug\", only: [:show], persisted: true\n```","ref":"Canary.Plugs.html#load_resource/2"},{"type":"extras","title":"Canary","doc":"Canary\n======\n[![Actions Status](https://github.com/cpjk/canary/workflows/CI/badge.svg)](https://github.com/runhyve/canary/actions?query=workflow%3ACI)\n[![Hex pm](https://img.shields.io/hexpm/v/canary.svg?style=flat)](https://hex.pm/packages/canary)\n\nAn authorization library in Elixir for Plug applications that restricts what resources\nthe current user is allowed to access, and automatically loads resources for the current request.\n\nInspired by [CanCan](https://github.com/CanCanCommunity/cancancan) for Ruby on Rails.\n\n[Read the docs](http://hexdocs.pm/canary)","ref":"readme.html"},{"type":"extras","title":"Installation - Canary","doc":"For the latest master:\n\n```elixir\ndefp deps do\n  {:canary, github: \"runhyve/canary\"}\nend\n```\n\nFor the latest release:\n\n```elixir\ndefp deps do\n  {:canary, \"~> 1.1.1\"}\nend\n```\n\nThen run `mix deps.get` to fetch the dependencies.","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Canary","doc":"Canary provides three functions to be used as plugs to load and authorize resources:\n\n`load_resource/2`, `authorize_resource/2`, and `load_and_authorize_resource/2`.\n\n`load_resource/2` and `authorize_resource/2` can be used by themselves, while `load_and_authorize_resource/2` combines them both.\n\nIn order to use Canary, you will need, at minimum:\n\n- A [Canada.Can protocol](https://github.com/jarednorman/canada) implementation (a good place would be `lib/abilities.ex`)\n\n- An Ecto record struct containing the user to authorize in `conn.assigns.current_user` (the key can be customized - see https://github.com/cpjk/canary#overriding-the-default-user).\n\n- Your Ecto repo specified in your `config/config.exs`: `config :canary, repo: YourApp.Repo`\n\nThen, just `import Canary.Plugs` in order to use the plugs. In a Phoenix app the best place would probably be inside `controller/0` in your `web/web.ex`, in order to make the functions available in all of your controllers.","ref":"readme.html#usage"},{"type":"extras","title":"load_resource/2 - Canary","doc":"Loads the resource having the id given in `conn.params[\"id\"]` from the database using the given Ecto repo and model, and assigns the resource to `conn.assigns. `, where `resource_name` is inferred from the model name.\n\nFor example,\n\n```elixir\nplug :load_resource, model: Project.Post\n```\nWill load the `Project.Post` having the id given in `conn.params[\"id\"]` through `YourApp.Repo`, into\n`conn.assigns.post`","ref":"readme.html#load_resource-2"},{"type":"extras","title":"authorize_resource/2 - Canary","doc":"Checks whether or not the `current_user` for the request can perform the given action on the given resource and assigns the result (true/false) to `conn.assigns.authorized`. It is up to you to decide what to do with the result.\n\nFor Phoenix applications, Canary determines the action automatically.\n\nFor non-Phoenix applications, or to override the action provided by Phoenix, simply ensure that `conn.assigns.canary_action` contains an atom specifying the action.\n\nIn order to authorize resources, you must specify permissions by implementing the [Canada.Can protocol](https://github.com/jarednorman/canada) for your `User` model (Canada is included as a light weight dependency).","ref":"readme.html#authorize_resource-2"},{"type":"extras","title":"load_and_authorize_resource/2 - Canary","doc":"Authorizes the resource and then loads it if authorization succeeds. Again, the resource is loaded into `conn.assigns. `.\n\nIn the following example, the `Post` with the same `user_id` as the `current_user` is only loaded if authorization succeeds.","ref":"readme.html#load_and_authorize_resource-2"},{"type":"extras","title":"Usage Example - Canary","doc":"Let's say you have a Phoenix application with a `Post` model, and you want to authorize the `current_user` for accessing `Post` resources.\n\nLet's suppose that you have a file named `lib/abilities.ex` that contains your Canada authorization rules like so:\n\n```elixir\ndefimpl Canada.Can, for: User do\n  def can?(%User{ id: user_id }, action, %Post{ user_id: user_id })\n    when action in [:show], do: true\n\n  def can?(%User{ id: user_id }, _, _), do: false\nend\n```\nand in your `web/router.ex:` you have:\n\n```elixir\nget \"/posts/:id\", PostController, :show\ndelete \"/posts/:id\", PostController, :delete\n```\n\nTo automatically load and authorize on the `Post` having the `id` given in the params, you would add the following plug to your `PostController`:\n\n```elixir\nplug :load_and_authorize_resource, model: Post\n```\n\nIn this case, on `GET /posts/12` authorization succeeds, and the `Post` specified by `conn.params[\"id]` will be loaded into `conn.assigns.post`.\n\nHowever, on `DELETE /posts/12`, authorization fails and the `Post` resource is not loaded.","ref":"readme.html#usage-example"},{"type":"extras","title":"Excluding actions - Canary","doc":"To exclude an action from any of the plugs, pass the `:except` key, with a single action or list of actions.\n\nFor example,\n\nSingle action form:\n\n```elixir\nplug :load_and_authorize_resource, model: Post, except: :show\n```\n\nList form:\n\n```elixir\nplug :load_and_authorize_resource, model: Post, except: [:show, :create]\n```","ref":"readme.html#excluding-actions"},{"type":"extras","title":"Authorizing only specific actions - Canary","doc":"To specify that a plug should be run only for a specific list of actions, pass the `:only` key, with a single action or list of actions.\n\nFor example,\n\nSingle action form:\n\n```elixir\nplug :load_and_authorize_resource, model: Post, only: :show\n```\n\nList form:\n\n```elixir\nplug :load_and_authorize_resource, model: Post, only: [:show, :create]\n```\n\nNote: Passing both `:only` and `:except` to a plug is invalid. Canary will simply pass the `Conn` along unchanged.","ref":"readme.html#authorizing-only-specific-actions"},{"type":"extras","title":"Overriding the default user - Canary","doc":"Globally, the default key for finding the user to authorize can be set in your configuration as follows:\n\n```elixir\nconfig :canary, current_user: :some_current_user\n```\n\nIn this case, canary will look for the current user record in `conn.assigns.some_current_user`.\n\nThe current user key can also be overridden for individual plugs as follows:\n\n```elixir\nplug :load_and_authorize_resource, model: Post, current_user: :current_admin\n```","ref":"readme.html#overriding-the-default-user"},{"type":"extras","title":"Specifying resource_name - Canary","doc":"To specify the name under which the loaded resource is stored, pass the `:as` flag in the plug declaration.\n\nFor example,\n\n```elixir\nplug :load_and_authorize_resource, model: Post, as: :new_post\n```\n\nwill load the post into `conn.assigns.new_post`","ref":"readme.html#specifying-resource_name"},{"type":"extras","title":"Preloading associations - Canary","doc":"Associations can be preloaded with `Repo.preload` by passing the `:preload` option with the name of the association:\n\n```elixir\nplug :load_and_authorize_resource, model: Post, preload: :comments\n```","ref":"readme.html#preloading-associations"},{"type":"extras","title":"Non-id actions - Canary","doc":"For the `:index`, `:new`, and `:create` actions, the resource passed to the `Canada.Can` implementation\nshould be the *module* name of the model rather than a struct.\n\nFor example, when authorizing access to the `Post` resource,\n\nyou should use\n\n```elixir\ndef can?(%User{}, :index, Post), do: true\n```\n\ninstead of\n\n```elixir\ndef can?(%User{}, :index, %Post{}), do: true\n```\n\nYou can specify additional actions for which Canary will authorize based on the model name, by passing the `non_id_actions` opt to the plug.\n\nFor example,\n```elixir\nplug :authorize_resource, model: Post, non_id_actions: [:find_by_name]\n```","ref":"readme.html#non-id-actions"},{"type":"extras","title":"Implementing Canada.Can for an anonymous user - Canary","doc":"You may wish to define permissions for when there is no logged in current user (when `conn.assigns.current_user` is `nil`).\nIn this case, you should implement `Canada.Can` for `nil` like so:\n\n```elixir\ndefimpl Canada.Can, for: Atom do\n  # When the user is not logged in, all they can do is read Posts\n  def can?(nil, :show, %Post{}), do: true\n  def can?(nil, _, _), do: false\nend\n```","ref":"readme.html#implementing-canada-can-for-an-anonymous-user"},{"type":"extras","title":"Nested associations - Canary","doc":"Sometimes you need to load and authorize a parent resource when you have a relationship between two resources and you are\ncreating a new one or listing all the children of that parent.  By specifying the `:persisted` option with `true`\nyou can load and/or authorize a nested resource.  Specifying this option overrides the default loading behavior of the\n`:index`, `:new`, and `:create` actions by loading an individual resource.  It also overrides the default\nauthorization behavior of the `:index`, `:new`, and `create` actions by loading a struct instead of a module\nname for the call to `Canada.can?`.\n\nFor example, when loading and authorizing a `Post` resource which can have one or more `Comment` resources, use\n\n```elixir\nplug :load_and_authorize_resource, model: Post, id_name: \"post_id\", persisted: true, only: [:create]\n```\n\nto load and authorize the parent `Post` resource using the `post_id` in /posts/:post_id/comments before you\ncreate the `Comment` resource using its parent.","ref":"readme.html#nested-associations"},{"type":"extras","title":"Specifing database field - Canary","doc":"You can tell Canary to search for a resource using a field other than the default `:id` by using the `:id_field` option. Note that the specified field must be able to uniquely identify any resource in the specified table.\n\nFor example, if you want to access your posts using a string field called `slug`, you can use\n\n```elixir\nplug :load_and_authorize_resource, model: Post, id_name: \"slug\", id_field: \"slug\"\n```\n\nto load and authorize the resource `Post` with the slug specified by `conn.params[\"slug\"]` value.\n\nIf you are using Phoenix, your `web/router.ex` should contain something like:\n\n```elixir\nresources \"/posts\", PostController, param: \"slug\"\n```\n\nThen your URLs will look like:\n\n```\n/posts/my-new-post\n```\n\ninstead of\n\n```\n/posts/1\n```","ref":"readme.html#specifing-database-field"},{"type":"extras","title":"Handling unauthorized actions - Canary","doc":"By default, when an action is unauthorized, Canary simply sets `conn.assigns.authorized` to `false`.\nHowever, you can configure a handler function to be called when authorization fails. Canary will pass the `Plug.Conn` to the given function. The handler should accept a `Plug.Conn` as its only argument, and should return a `Plug.Conn`.\n\nFor example, to have Canary call `Helpers.handle_unauthorized/1`:\n\n```elixir\nconfig :canary, unauthorized_handler: {Helpers, :handle_unauthorized}\n```","ref":"readme.html#handling-unauthorized-actions"},{"type":"extras","title":"Handling resource not found - Canary","doc":"By default, when a resource is not found, Canary simply sets the resource in `conn.assigns` to `nil`. Like unauthorized action handling , you can configure a function to which Canary will pass the `conn` when a resource is not found:\n\n```elixir\nconfig :canary, not_found_handler: {Helpers, :handle_not_found}\n```\n\nYou can also specify handlers on an individual basis (which will override the corresponding configured handler, if any) by specifying the corresponding `opt` in the plug call:\n\n```elixir\nplug :load_and_authorize_resource Post,\n  unauthorized_handler: {Helpers, :handle_unauthorized},\n  not_found_handler: {Helpers, :handle_not_found}\n```\n\nTip: If you would like the request handling to stop after the handler function exits, e.g. when redirecting, be sure to call `Plug.Conn.halt/1` within your handler like so:\n\n```elixir\ndef handle_unauthorized(conn) do\n  conn\n  |> put_flash(:error, \"You can't access that page!\")\n  |> redirect(to: \"/\")\n  |> halt\nend\n```\n\nNote: If both an `:unauthorized_handler` and a `:not_found_handler` are specified for `load_and_authorize_resource`, and the request meets the criteria for both, the `:unauthorized_handler` will be called first.","ref":"readme.html#handling-resource-not-found"},{"type":"extras","title":"License - Canary","doc":"MIT License. Copyright 2016 Chris Kelly.","ref":"readme.html#license"}],"content_type":"text/markdown","proglang":"elixir","producer":{"name":"ex_doc","version":"0.36.1"}}